#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "make_property4xi.py">
#<abstract4xi abstract = "Module providing function make_property4xi"/>
#<refer4xi reference = "sharedctypes.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'make_property4xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import weakref         
except:
    pass
try:                 # rare ones
    import ctypes          
except:
    pass 
try:                 # imported-as ones 
    pass          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    pass
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from print4xi import print4xi
from time4xi import time4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global prop_cache
global class_cache
#"
#/>
 
# -------------------------------------------------------------------------------------------------
# Settings
#
# sharedctypes.py:
#      scls = class_cache[cls] = type(classname, (SynchronizedBase,), d)
#      prop_cache[name] = d[name]
#      value = make_property('value')
#      raw = make_property('raw')
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
prop_cache = {}
class_cache = weakref.WeakKeyDictionary()
#"
#"/>


# -------------------------------------------------------------------------------------------------
#                               
# -------------------------------------------------------------------------------------------------
template = '''
def get%s(self):
    self.acquire()
    try:
        return self._obj.%s
    finally:
        self.release()
# -------------------------------------------------------------------------------------------------
#                               
# -------------------------------------------------------------------------------------------------        
def set%s(self, value):
    self.acquire()
    try:
        self._obj.%s = value
    finally:
        self.release()
%s = property(get%s, set%s)
'''
 
                        


# -------------------------------------------------------------------------------------------------
#<function4xi function = "make-property4xi" usage = "
#   sharedctypes.py                             
#       Synchronized(SynchronizedBase):
#           value = make_property('value')
#           d = dict((name, make_property(name))
#   synchronized4xi.py:
#       value = make_property4xi('value')
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def make_property4xi(name):
    '''
    make_property4xi function in sharedctypes.py class Synchronized: value = make_property('value') 
    '''
    global prop_cache
    global class_cache        
    
    # ---------------------------------------------------------------------------------------------    
    #
    # --------------------------------------------------------------------------------------------- 
        
    class2print = "make_property4xi"
    method2print = "make_property4xi"
    
    message2print = "{started" + "; make-property4xi-started: " + str(make_property4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
        
    message2print = "{passed" + "; name-passed-to-make-property4xi: " + str(name) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
                  
        
    # ---------------------------------------------------------------------------------------------    
    #
    # --------------------------------------------------------------------------------------------- 
    message2print = "{using" + "; prop-cache-using-by-make-property4xi: " + str(prop_cache)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    

    message2print = "{using" + "; class-cache-using-by-make-property4xi: " + str(class_cache)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
        
    try:
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        
        message2print = "{returning" + "; prop-cache-name-returning-by-make-property4xi: " + str(prop_cache[name]) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   

        return prop_cache[name]
        
    except KeyError:
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        
        d2xi = {}
        
        message2print = "{passing" + "; dictionary2xi-passing-to-exec-by-make-property4xi: " + str(d2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        message2print = "{passing" + "; name-passing-to-exec-by-make-property4xi: " + str(name) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
          
        exec(template % ((name,)*7), d2xi)
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
               
        prop_cache[name] = d2xi[name]
        
        message2print = "{set" + "; prop-cache-set-by-make-property4xi to: " + str(prop_cache[name])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{returning" + "; dictionary2xi-element-returning-by-make-property4xi: " + str(d2xi[name]) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        
                
        return d2xi[name]    
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of make_property4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

